/*
********************************************************************************
* COPYRIGHT(c)    ϻ, 2016
* 
*        (as is).
*     .
********************************************************************************
*/


#include "TM10P_board.h"



static volatile uint8_t State = 0;
static volatile uint8_t ADCBufState = 0;
static uint8_t DataToShow[COLS_NUM] = {0, 0, 0, 0, 0};
static int8_t CurPeakPos[COLS_NUM] = {0, 0, 0, 0, 0};

static q15_t ADCBuf_1[2 * NUM_OF_ADC_VALUES];
static q15_t ADCBuf_2[2 * NUM_OF_ADC_VALUES];
static q15_t *ADCData = ADCBuf_1;
static q15_t *InputData = ADCBuf_2;
static q15_t OutputData[2 * NUM_OF_ADC_VALUES];


int main()
{   
  TM10P_Init();
  
  arm_rfft_instance_q15 S;
  arm_rfft_init_q15(&S, NUM_OF_ADC_VALUES, 0, 1);
  
  
  for(;;)
  { 
    for (uint16_t i = 0; i < NUM_OF_ADC_VALUES; i++)
      //InputData[i] = (q15_t)(4 * (InputData[i] - 2048) * WindowCoef[i]);
      //InputData[i] = (q15_t)(2 * (InputData[i] - 2048) * WindowCoef[i]);
      InputData[i] = (q15_t)((InputData[i] - 2048) * WindowCoef[i]);

    for (uint16_t i = (NUM_OF_ADC_VALUES - 1); i >= 1; i--)
    {
      InputData[2 * i] = InputData[i];
      InputData[2 * i + 1] = 0;
    }
    InputData[1] = 0;
     
    arm_rfft_q15(&S, InputData, OutputData);
  
    for (uint16_t i = 0; i < (2 * NUM_OF_ADC_VALUES); i++)
    {
      uint16_t Sign = InputData[i] & 0x8000;
      InputData[i] = InputData[i] << 7;
      InputData[i] &= ~0x8000;
      InputData[i] |= Sign;
    }
      
    arm_cmplx_mag_q15(InputData, OutputData, NUM_OF_ADC_VALUES);
    
    for (uint16_t i = 0; i < (NUM_OF_ADC_VALUES / 2); i++)
    {
      if (OutputData[i] < THRESHOLD_COEF)
        OutputData[i] = 0;
    }
      
    uint8_t NewData[COLS_NUM];
      
    for (uint8_t i = 0; i < COLS_NUM; i++)
    {
/*      
    //   
      uint8_t FreqIndex[COLS_NUM] = {COL_1_FREQ, COL_2_FREQ, COL_3_FREQ, COL_4_FREQ, COL_5_FREQ};
      float32_t FreqCoef[COLS_NUM] = {2.0f, 0.5f, 0.5f, 0.25f, 0.15f};
        
      NewData[i] = (uint8_t)(OutputData[FreqIndex[i]] / (THRESHOLD_COEF * FreqCoef[i]));
*/
      
      //   
      float32_t BandCoef[COLS_NUM] = {1.0f, 0.25f, 0.25f, 0.025f, 0.07f};
      uint8_t BandsStart[COLS_NUM] = {BAND_1_FIRST_BIN, BAND_2_FIRST_BIN, BAND_3_FIRST_BIN, BAND_4_FIRST_BIN, BAND_5_FIRST_BIN};
      uint8_t BandsEnd[COLS_NUM] = {BAND_1_LAST_BIN, BAND_2_LAST_BIN, BAND_3_LAST_BIN, BAND_4_LAST_BIN, BAND_5_LAST_BIN};
      
      uint32_t BandMagSum = 0;
      
      for (uint8_t j = BandsStart[i]; j <= BandsEnd[i]; j++)
        BandMagSum += OutputData[j];

       NewData[i] = (uint8_t)(BandMagSum / (THRESHOLD_COEF * BandCoef[i]));

      if (NewData[i] > ROWS_NUM)
        NewData[i] = ROWS_NUM;
    }
      
    while((State & END_OF_INDICATION) != END_OF_INDICATION);
    TIME_BASE_TIMER->CR1 &= ~TIM_CR1_CEN;
    
    for(uint8_t i = 0; i < COLS_NUM; i++)
    {
      DataToShow[i] = NewData[i];
    }
    
    State &= ~END_OF_INDICATION;
    TIME_BASE_TIMER->CR1 |= TIM_CR1_CEN;
    
    while((ADCBufState & ADC_BUFFER_FULL) != ADC_BUFFER_FULL);

    if (ADCData == ADCBuf_1)
    {
      ADCData = ADCBuf_2;
      InputData = ADCBuf_1;
    }
    else
    {
      ADCData = ADCBuf_1;
      InputData = ADCBuf_2;
    }
  
    ADCBufState &= ~ADC_BUFFER_FULL;
  
    DMA_CNL_FOR_ADC->CMAR = (uint32_t)(ADCData);
    DMA_CNL_FOR_ADC->CNDTR = NUM_OF_ADC_VALUES;
    DMA_CNL_FOR_ADC->CCR |= DMA_CCR_EN;
  
    ADC_TIMER->CR1 |= TIM_CR1_CEN;
  }      
}
//------------------------------------------------------------------------------
void TM10P_Init()
{
  //HSI, PLL, 48 MHz
  FLASH->ACR = FLASH_ACR_PRFTBE | (uint32_t)FLASH_ACR_LATENCY;
  // HCLK = SYSCLK / 1
  RCC->CFGR |= (uint32_t)RCC_CFGR_HPRE_DIV1;
  // PCLK2 = HCLK / 1
  RCC->CFGR |= (uint32_t)RCC_CFGR_PPRE_DIV1;
  //  PLL: (HSI / 2) * 12 = (8 / 2) * 12 = 48 ()
  RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_PLLSRC | RCC_CFGR_PLLXTPRE | RCC_CFGR_PLLMUL));
  RCC->CFGR |= (uint32_t)(RCC_CFGR_PLLSRC_HSI_DIV2 | RCC_CFGR_PLLMUL12);

  RCC->CR |= RCC_CR_PLLON;

  while((RCC->CR & RCC_CR_PLLRDY) == 0)
  {
  }

  RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_SW));
  RCC->CFGR |= (uint32_t)RCC_CFGR_SW_PLL;

  while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != (uint32_t)RCC_CFGR_SWS_PLL)
  {
  }
  
  
  RCC->AHBENR |= RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN | RCC_AHBENR_GPIOFEN;
  
  GPIOA->MODER &= ~((3 << (2 * ROW1_PIN)) | (3 << (2 * ROW2_PIN)) | (3 << (2 * ROW3_PIN)) | \
                    (3 << (2 * ROW4_PIN)) | (3 << (2 * ROW5_PIN)) | (3 << (2 * ROW6_PIN)) | \
                    (3 << (2 * ROW7_PIN)) | (3 << (2 * COL1_PIN)) | (3 << (2 * COL2_PIN)));
  
  GPIOA->MODER |=  ((1 << (2 * ROW1_PIN)) | (1 << (2 * ROW2_PIN)) | (1 << (2 * ROW3_PIN)) | \
                    (1 << (2 * ROW4_PIN)) | (1 << (2 * ROW5_PIN)) | (1 << (2 * ROW6_PIN)) | \
                    (1 << (2 * ROW7_PIN)) | (1 << (2 * COL1_PIN)) | (1 << (2 * COL2_PIN)));
 
  GPIOA->PUPDR &= ~((3 << (2 * COL1_PIN)) | (3 << (2 * COL2_PIN)));
  
  GPIOB->MODER &= ~(3 << (2 * COL5_PIN));
  GPIOB->MODER |=  (1 << (2 * COL5_PIN));
  
  GPIOF->MODER &= ~((3 << (2 * COL3_PIN)) | (3 << (2 * COL4_PIN)));
  GPIOF->MODER |=  ((1 << (2 * COL3_PIN)) | (1 << (2 * COL4_PIN)));
  
  //
  ADC_GPIO->MODER |= 3 << (2 * ADC_PIN);
  RCC->ADC_ENR |= ADC_CLK_EN;
  
  ADC_USED->CR |= ADC_CR_ADCAL;
  while(((ADC_USED->CR) & ADC_CR_ADCAL) == ADC_CR_ADCAL);
  
  ADC_USED->CFGR2 |= ADC_CFGR2_CKMODE_0;
  ADC_USED->SMPR |= ADC_SMPR_SMP_0;
  ADC_USED->CHSELR |= ADC_CHSELR_CHSEL0;
  ADC_USED->IER |= ADC_IER_OVRIE;
  ADC_USED->CFGR1 |= ADC_CFGR1_EXTEN_0 | ADC_CFGR1_EXTSEL_0 | ADC_CFGR1_EXTSEL_1 \
                   | ADC_CFGR1_DMAEN | ADC_CFGR1_DMACFG;
  ADC_USED->CR |= ADC_CR_ADEN;
  while(((ADC_USED->ISR) & ADC_ISR_ADRDY) != ADC_ISR_ADRDY);
  NVIC_EnableIRQ(ADC_IRQ);
  NVIC_SetPriority(ADC_IRQ, 0);

    
  // (DMA)
  RCC->DMA_FOR_ADC_ENR |= DMA_FOR_ADC_CLK_EN;
  DMA_CNL_FOR_ADC->CPAR = (uint32_t) (&(ADC_USED->DR));
  DMA_CNL_FOR_ADC->CMAR = (uint32_t)(ADCData);
  DMA_CNL_FOR_ADC->CNDTR = NUM_OF_ADC_VALUES;
  DMA_CNL_FOR_ADC->CCR |= DMA_CCR_MINC | DMA_CCR_MSIZE_0 | DMA_CCR_PSIZE_0 \
                        | DMA_CCR_TEIE | DMA_CCR_TCIE; 
  DMA_CNL_FOR_ADC->CCR |= DMA_CCR_EN;
  NVIC_EnableIRQ(DMA_CNL_FOR_ADC_IRQ);
  NVIC_SetPriority(DMA_CNL_FOR_ADC_IRQ,0);

  
  //   :  = 1
  RCC->TIME_BASE_TIMER_ENR |= TIME_BASE_TIMER_CLK_EN;
  TIME_BASE_TIMER->PSC = 959;
  TIME_BASE_TIMER->ARR = 49;
  TIME_BASE_TIMER->DIER = TIM_DIER_UIE;
  NVIC_EnableIRQ(TIME_BASE_TIMER_IRQ);
  NVIC_SetPriority(TIME_BASE_TIMER_IRQ, 1);
  
  //   :  = 40000 
  RCC->ADC_TIMER_ENR |= ADC_TIMER_CLK_EN;
  ADC_TIMER->CR2 |= TIM_CR2_MMS_1;
  ADC_TIMER->PSC = 119;
  ADC_TIMER->ARR = 9;
    
  
  TIME_BASE_TIMER->CR1 |= TIM_CR1_CEN;
  ADC_TIMER->CR1 |= TIM_CR1_CEN;
  ADC_USED->CR |= ADC_CR_ADSTART;
}
//------------------------------------------------------------------------------
void TIME_BASE_TIMER_ISR(void)
{
  GPIO_TypeDef* COLS_GPIOS[COLS_NUM] = {COL1_GPIO, COL2_GPIO, COL3_GPIO, COL4_GPIO, COL5_GPIO};
  uint16_t COLS_PINS[COLS_NUM] = {COL1_PIN, COL2_PIN, COL3_PIN, COL4_PIN, COL5_PIN};
  uint16_t ROWS_PINS[ROWS_NUM] = {ROW1_PIN, ROW2_PIN, ROW3_PIN, ROW4_PIN, ROW5_PIN, ROW6_PIN, ROW7_PIN};
  
  static uint8_t CurPos[COLS_NUM] = {0, 0, 0, 0, 0};
  static uint8_t CurCol = 0;
  static uint8_t UpdateCnt = 0;
  static uint8_t UpdatePeakCnt = 0;
  const uint8_t UpdateValue = 4;
  
  TIME_BASE_TIMER->SR &= ~TIM_SR_UIF;
  
  COL1_GPIO->ODR &= ~((1 << COL1_PIN) | (1 << COL2_PIN) | ROWS_MASK);
  COL3_GPIO->ODR &= ~((1 << COL3_PIN) | (1 << COL4_PIN));
  COL5_GPIO->ODR &= ~(1 << COL5_PIN);
  
  
  if (CurPos[CurCol] <= DataToShow[CurCol])
  {
    if (DataToShow[CurCol] != 0)
    {
      if ((State & SINGLE_LED_MODE) != SINGLE_LED_MODE)
      {
        for (uint8_t i = 0; i < CurPos[CurCol]; i++)
          ROWS_GPIO->ODR |= 1 << ROWS_PINS[i];
      }
      else
        ROWS_GPIO->ODR |= 1 << ROWS_PINS[CurPos[CurCol] - 1];
    }
      
    if (CurPos[CurCol] != DataToShow[CurCol])
    {
      if(UpdateCnt == UpdateValue)
        CurPos[CurCol]++;
    }

    else if ((State & (1 << CurCol)) != (1 << CurCol))
    {
      State |= (1 << CurCol);
      
      if ((State & SINGLE_LED_MODE) != SINGLE_LED_MODE)
      {
        if ((DataToShow[CurCol] != 0) && ((CurPeakPos[CurCol] < 0) || (CurPeakPos[CurCol] < DataToShow[CurCol])))
          CurPeakPos[CurCol] = DataToShow[CurCol];
      }
    }
  }
  
  else if (CurPos[CurCol] > DataToShow[CurCol])
  {
    if ((State & SINGLE_LED_MODE) != SINGLE_LED_MODE)
    {
      for (uint8_t i = 0; i < CurPos[CurCol]; i++)
        ROWS_GPIO->ODR |= (1 << ROWS_PINS[i]);
    }
    else
      ROWS_GPIO->ODR |= 1 << ROWS_PINS[CurPos[CurCol] - 1];

    if (UpdateCnt == UpdateValue)
      CurPos[CurCol]--;
  }
  
  if ((State & SINGLE_LED_MODE) != SINGLE_LED_MODE)
  {
    if (CurPeakPos[CurCol] >= 0)
    {
      ROWS_GPIO->ODR |= 1 << ROWS_PINS[CurPeakPos[CurCol]];
      
      if (UpdatePeakCnt == (3 * UpdateValue))
        CurPeakPos[CurCol]--;      
    }
  }
  
  
  COLS_GPIOS[CurCol]->ODR |= 1 << COLS_PINS[CurCol];
  
  CurCol++;
  if (CurCol == COLS_NUM)
  {
    CurCol = 0;
    UpdateCnt++;
    if (UpdateCnt == (UpdateValue + 1))
      UpdateCnt = 0;
    
    if ((State & SINGLE_LED_MODE) != SINGLE_LED_MODE)
    {
      UpdatePeakCnt++;
      if (UpdatePeakCnt == (3 * UpdateValue + 1))
        UpdatePeakCnt = 0;
    }
  }
}
//------------------------------------------------------------------------------
void ADC_ISR(void)
{
  if ((ADC1->ISR & ADC_ISR_OVR) == ADC_ISR_OVR) 
  {
    ADC1->ISR |= ADC_ISR_OVR;
  }
}
//------------------------------------------------------------------------------
void DMA_FOR_ADC_ISR(void)
{
  if ((DMA_FOR_ADC->ISR & DMA_ISR_TEIF1) == DMA_ISR_TEIF1)
  {
    DMA_FOR_ADC->IFCR |= DMA_IFCR_CTEIF1;
    DMA_CNL_FOR_ADC->CCR |= DMA_CCR_EN;
  }
  
  else if ((DMA_FOR_ADC->ISR & DMA_CCR_TCIE) == DMA_CCR_TCIE)
  {
    ADC_TIMER->CR1 &= ~TIM_CR1_CEN;
    ADCBufState |= ADC_BUFFER_FULL;
    DMA_FOR_ADC->IFCR |= DMA_CCR_TCIE | DMA_CCR_HTIE;
    DMA_CNL_FOR_ADC->CCR &= ~DMA_CCR_EN;
  }
}

